@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 
-# $Id: test_io.py 5889 2009-04-01 20:00:21Z gbrandl $
+# $Id: test_io.py 7037 2011-05-19 08:56:27Z milde $
 # Author: Lea Wiemann <LeWiemann@gmail.com>
 # Copyright: This module has been placed in the public domain.
 
@@ -8,10 +8,10 @@
 Test module for io.py.
 """
 
-import unittest
+import unittest, sys
 import DocutilsTestSupport              # must be imported before docutils
 from docutils import io
-from docutils._compat import b
+from docutils._compat import b, bytes
 
 
 class InputTests(unittest.TestCase):
