@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 
-# $Id: test_SimpleTableParser.py 4564 2006-05-21 20:44:42Z wiemann $
+# $Id: test_SimpleTableParser.py 7074 2011-07-07 06:54:37Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -40,6 +40,16 @@ and           two rows.
   [[0, 0, 2, ['and']],
    [0, 0, 2, ['two rows.']]]])],
 ["""\
+======================================
+The last row might stick into the margin
+second row.
+======================================
+""",
+([40],
+ [],
+ [[[0, 0, 1, ['The last row might stick into the margin']]],
+  [[0, 0, 2, ['second row.']]]])],
+["""\
 ==========  ===========
 A table with four rows,
 -----------------------
@@ -64,6 +74,13 @@ cell 3          cell 4
 """,
 'TableMarkupError: Text in column margin at line offset 1.'],
 ["""\
+======  =====  ======
+row     one
+Another bad    table
+======  =====  ======
+""",
+'TableMarkupError: Text in column margin at line offset 2.'],
+["""\
 ===========  ================
 A table with two header rows,
 -----------------------------
