@@ -1,6 +1,7 @@
+# -*- coding: utf-8 -*-
 #! /usr/bin/env python
 
-# $Id: test_utils.py 6271 2010-03-22 14:57:16Z milde $
+# $Id: test_utils.py 7007 2011-04-12 09:10:57Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -11,10 +12,10 @@ Test module for utils.py.
 import unittest
 import sys
 from DocutilsTestSupport import utils, nodes
-if sys.version_info < (3,0):
-    from StringIO import StringIO
-else:
+try:
     from io import StringIO
+except ImportError:    # io is new in Python 2.6
+    from StringIO import StringIO
 
 
 class ReporterTests(unittest.TestCase):
@@ -72,6 +73,28 @@ class ReporterTests(unittest.TestCase):
                           'a severe error, raises an exception\n')
 
 
+    def test_unicode_message(self):
+        sw = self.reporter.system_message(0, u'mesidʒ')
+        self.assertEquals(sw.pformat(), u"""\
+<system_message level="0" source="test data" type="DEBUG">
+    <paragraph>
+        mesidʒ
+""")
+
+    def test_unicode_message_from_exception(self):
+        """Workaround for Python < 2.6 bug:
+        unicode(<exception instance>) uses __str__
+        and hence fails with unicode message"""
+        try:
+            raise Exception(u'mesidʒ')
+        except Exception, err:
+            sw = self.reporter.system_message(0, err)
+            self.assertEquals(sw.pformat(), u"""\
+<system_message level="0" source="test data" type="DEBUG">
+    <paragraph>
+        mesidʒ
+""")
+
 class QuietReporterTests(unittest.TestCase):
 
     stream = StringIO()
@@ -211,5 +234,23 @@ class ExtensionOptionTests(unittest.TestCase):
                           field_list, self.optionspec)
 
 
+class HelperFunctionsTests(unittest.TestCase):
+
+    def test_normalize_language_tag(self):
+        self.assertEquals(utils.normalize_language_tag('de'), ['de'])
+        self.assertEquals(utils.normalize_language_tag('de-AT'),
+                          ['de_at', 'de'])
+        self.assertEquals(utils.normalize_language_tag('de-AT-1901'),
+                          ['de_at_1901', 'de_at', 'de_1901', 'de'])
+        self.assertEquals(utils.normalize_language_tag('de-AT-1901-frak'),
+                          ['de_at_1901_frak', 'de_at_1901', 'de_at_frak',
+                          'de_1901_frak', 'de_at', 'de_1901', 'de_frak', 'de'])
+
+    def test_column_width(self):
+        self.assertEquals(utils.column_width(u'de'), 2)
+        self.assertEquals(utils.column_width(u'dâ'), 2) # pre-composed
+        self.assertEquals(utils.column_width(u'dâ'), 2) # combining
+
+
 if __name__ == '__main__':
     unittest.main()
