@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# $Id: buildhtml.py 5741 2008-12-01 07:21:03Z grubert $
+# $Id: buildhtml.py 7037 2011-05-19 08:56:27Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -29,6 +29,7 @@ from fnmatch import fnmatch
 import docutils
 from docutils import ApplicationError
 from docutils import core, frontend, utils
+from docutils.error_reporting import ErrorOutput, ErrorString
 from docutils.parsers import rst
 from docutils.readers import standalone, pep
 from docutils.writers import html4css1, pep_html
@@ -58,12 +59,12 @@ class SettingsSpec(docutils.SettingsSpec):
            'validator': frontend.validate_boolean}),
          ('Do not scan subdirectories for files to process.',
           ['--local'], {'dest': 'recurse', 'action': 'store_false'}),
-         ('Do not process files in <directory>.  This option may be used '
+         ('BROKEN Do not process files in <directory>.  This option may be used '
           'more than once to specify multiple directories.',
           ['--prune'],
           {'metavar': '<directory>', 'action': 'append',
            'validator': frontend.validate_colon_separated_string_list}),
-         ('Recursively ignore files or directories matching any of the given '
+         ('BROKEN Recursively ignore files or directories matching any of the given '
           'wildcard (shell globbing) patterns (separated by colons).  '
           'Default: ".svn:CVS"',
           ['--ignore'],
@@ -72,6 +73,9 @@ class SettingsSpec(docutils.SettingsSpec):
            'validator': frontend.validate_colon_separated_string_list}),
          ('Work silently (no progress messages).  Independent of "--quiet".',
           ['--silent'],
+          {'action': 'store_true', 'validator': frontend.validate_boolean}),
+         ('Do not process files, show files that would be processed.',
+          ['--dry-run'],
           {'action': 'store_true', 'validator': frontend.validate_boolean}),))
 
     relative_path_settings = ('prune',)
@@ -181,22 +185,25 @@ class Builder:
         else:
             self.directories = [os.getcwd()]
         for directory in self.directories:
-            try:
-                for root, dirs, files in os.walk(directory):
-                    self.visit(recurse, root, dirs+files)
-            except (AttributeError): # python2.2 does not have os.walk
-                print "no os.walk"
-                os.path.walk(directory, self.visit, recurse)
+            for root, dirs, files in os.walk(directory):
+                # os.walk by default this recurses down the tree,
+                # influence by modifying dirs.
+                if not recurse:
+                    del dirs[:]
+                self.visit(root, files)
 
-    def visit(self, recurse, directory, names):
+    def visit(self, directory, names):
+        # BUG prune and ignore do not work 
         settings = self.get_settings('', directory)
+        errout = ErrorOutput(encoding=settings.error_encoding)
         if settings.prune and (os.path.abspath(directory) in settings.prune):
-            print >>sys.stderr, '/// ...Skipping directory (pruned):', directory
+            print >>errout, ('/// ...Skipping directory (pruned): %s' %
+                              directory)
             sys.stderr.flush()
             names[:] = []
             return
         if not self.initial_settings.silent:
-            print >>sys.stderr, '/// Processing directory:', directory
+            print >>errout, '/// Processing directory: %s' % directory
             sys.stderr.flush()
         # settings.ignore grows many duplicate entries as we recurse
         # if we add patterns in config files or on the command line.
@@ -211,8 +218,6 @@ class Builder:
                 prune = self.process_txt(directory, name)
                 if prune:
                     break
-        if not recurse:
-            del names[:]
 
     def process_txt(self, directory, name):
         if name.startswith('pep-'):
@@ -220,24 +225,25 @@ class Builder:
         else:
             publisher = '.txt'
         settings = self.get_settings(publisher, directory)
+        errout = ErrorOutput(encoding=settings.error_encoding)
         pub_struct = self.publishers[publisher]
         if settings.prune and (directory in settings.prune):
             return 1
         settings._source = os.path.normpath(os.path.join(directory, name))
         settings._destination = settings._source[:-4]+'.html'
         if not self.initial_settings.silent:
-            print >>sys.stderr, '    ::: Processing:', name
+            print >>errout, '    ::: Processing: %s' % name
             sys.stderr.flush()
         try:
-            core.publish_file(source_path=settings._source,
+            if not settings.dry_run:
+                core.publish_file(source_path=settings._source,
                               destination_path=settings._destination,
                               reader_name=pub_struct.reader_name,
                               parser_name='restructuredtext',
                               writer_name=pub_struct.writer_name,
                               settings=settings)
         except ApplicationError, error:
-            print >>sys.stderr, ('        Error (%s): %s'
-                                 % (error.__class__.__name__, error))
+            print >>errout, '        %s' % ErrorString(error)
 
 
 if __name__ == "__main__":
