@@ -1,6 +1,6 @@
 #!/usr/bin/python -i
 
-# $Id: profile_docutils.py 5174 2007-05-31 00:01:52Z wiemann $
+# $Id: profile_docutils.py 5618 2008-07-28 08:37:32Z strank $
 # Author: Lea Wiemann <LeWiemann@gmail.com>
 # Copyright: This script has been placed in the public domain.
 
@@ -34,6 +34,6 @@ stats.sort_stats('time')
 stats.print_stats(40)
 
 try:
-    execfile(os.environ['PYTHONSTARTUP'])
+    exec(open(os.environ['PYTHONSTARTUP']).read())
 except:
     pass
